@@ -3,6 +3,7 @@
  *
  * Written by
  *  Andreas Boose <viceteam@t-online.de>
+ *  Marco van den Heuvel <blackystardust68@yahoo.com>
  *
  * This file is part of VICE, the Versatile Commodore Emulator.
  * See README for copyright notice.
@@ -35,6 +36,8 @@
 #include "c64meminit.h"
 #include "c64memrom.h"
 #include "cartio.h"
+#include "machine.h"
+#include "resources.h"
 #include "sid.h"
 #include "vicii-mem.h"
 
@@ -44,14 +47,14 @@
 
  bit 4 - !game
  bit 3 - !exrom
- bit 2 - loram
+ bit 2 - charen
  bit 1 - hiram
- bit 0 - charen
+ bit 0 - loram
 
          8000      a000      d000      e000
 
  0 0x00
- 1 0x01                      chr 
+ 1 0x01                      chr
  2 0x02                      chr       ker
  3 0x03            bas       chr       ker
  4 0x04
@@ -63,7 +66,7 @@
  9 0x09                      chr
 10 0x0a            romh      chr       ker
 11 0x0b  roml      bas/romh  chr       ker      8k game
-12 0x0c  
+12 0x0c
 13 0x0d                      io
 14 0x0e            romh      io        ker
 15 0x0f  roml      bas/romh  io        ker      8k game
@@ -78,7 +81,7 @@
 23 0x17  roml      -         io        romh     ultimax
 
 24 0x18
-25 0x19                      chr
+25 0x19
 26 0x1a            romh      chr       ker
 27 0x1b  roml      romh      chr       ker      16k game
 28 0x1c
@@ -97,7 +100,7 @@ const unsigned int c64meminit_io_config[32] = {
 };
 
 /* ROML is enabled at memory configs 11, 15, 27, 31 and Ultimax.  */
-static const unsigned int c64meminit_roml_config[32] = {
+const unsigned int c64meminit_roml_config[32] = {
     0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 1, 0, 0, 0, 1,
     1, 1, 1, 1, 1, 1, 1, 1,
@@ -106,7 +109,7 @@ static const unsigned int c64meminit_roml_config[32] = {
 
 /* ROMH is enabled at memory configs 10, 11, 14, 15, 26, 27, 30, 31
    and Ultimax.  */
-static const unsigned int c64meminit_romh_config[32] = {
+const unsigned int c64meminit_romh_config[32] = {
     0, 0, 0, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0,
     1, 1, 1, 1, 1, 1, 1, 1,
@@ -115,31 +118,34 @@ static const unsigned int c64meminit_romh_config[32] = {
 
 /* ROMH is mapped to $A000-$BFFF at memory configs 10, 11, 14, 15, 26,
    27, 30, 31.  If Ultimax is enabled it is mapped to $E000-$FFFF.  */
-static const unsigned int c64meminit_romh_mapping[32] = {
-    0x00, 0x00, 0x00, 0x00,
-    0x00, 0x00, 0x00, 0x00,
-    0x00, 0x00, 0x00, 0x00,
-    0x00, 0x00, 0x00, 0x00,
-    0xe0, 0xe0, 0xe0, 0xe0,
-    0xe0, 0xe0, 0xe0, 0xe0,
-    0x00, 0x00, 0xa0, 0xa0,
-    0x00, 0x00, 0xa0, 0xa0
+const unsigned int c64meminit_romh_mapping[32] = {
+    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+    0xe0, 0xe0, 0xe0, 0xe0, 0xe0, 0xe0, 0xe0, 0xe0,
+    0x00, 0x00, 0xa0, 0xa0, 0x00, 0x00, 0xa0, 0xa0
 };
 
 void c64meminit(unsigned int base)
 {
     unsigned int i, j;
+    int board = 0;
 
-    /* Setup BASIC ROM at $A000-$BFFF (memory configs 3, 7, 11, 15).  */
-    for (i = 0xa0; i <= 0xbf; i++) {
-        mem_read_tab_set(base + 3, i, c64memrom_basic64_read);
-        mem_read_tab_set(base + 7, i, c64memrom_basic64_read);
-        mem_read_tab_set(base + 11, i, c64memrom_basic64_read);
-        mem_read_tab_set(base + 15, i, c64memrom_basic64_read);
-        mem_read_base_set(base + 3, i, c64memrom_basic64_rom - 0xa000);
-        mem_read_base_set(base + 7, i, c64memrom_basic64_rom - 0xa000);
-        mem_read_base_set(base + 11, i, c64memrom_basic64_rom - 0xa000);
-        mem_read_base_set(base + 15, i, c64memrom_basic64_rom - 0xa000);
+    if (machine_class != VICE_MACHINE_C128) {
+        resources_get_int("BoardType", &board);
+    }
+
+    if (board != 1) {
+        /* Setup BASIC ROM at $A000-$BFFF (memory configs 3, 7, 11, 15).  */
+        for (i = 0xa0; i <= 0xbf; i++) {
+            mem_read_tab_set(base + 3, i, c64memrom_basic64_read);
+            mem_read_tab_set(base + 7, i, c64memrom_basic64_read);
+            mem_read_tab_set(base + 11, i, c64memrom_basic64_read);
+            mem_read_tab_set(base + 15, i, c64memrom_basic64_read);
+            mem_read_base_set(base + 3, i, c64memrom_basic64_rom - 0xa000);
+            mem_read_base_set(base + 7, i, c64memrom_basic64_rom - 0xa000);
+            mem_read_base_set(base + 11, i, c64memrom_basic64_rom - 0xa000);
+            mem_read_base_set(base + 15, i, c64memrom_basic64_rom - 0xa000);
+        }
     }
 
     /* Setup I/O at $D000-$DFFF (memory configs 5, 6, 7).  */
@@ -168,8 +174,9 @@ void c64meminit(unsigned int base)
 
             mem_read_tab_set(base + j, 0xdc, cia1_read);
             mem_set_write_hook(base + j, 0xdc, cia1_store);
-            mem_read_tab_set(base + j, 0xdd, cia2_read);
-            mem_set_write_hook(base + j, 0xdd, cia2_store);
+
+            mem_read_tab_set(base + j, 0xdd, c64io_dd00_read);
+            mem_set_write_hook(base + j, 0xdd, c64io_dd00_store);
 
             mem_read_tab_set(base + j, 0xde, c64io_de00_read);
             mem_set_write_hook(base + j, 0xde, c64io_de00_store);
@@ -189,33 +196,35 @@ void c64meminit(unsigned int base)
         }
     }
 
-    /* Setup Kernal ROM at $E000-$FFFF (memory configs 2, 3, 6, 7, 10,
-       11, 14, 15, 26, 27, 30, 31).  */
-    for (i = 0xe0; i <= 0xff; i++) {
-        mem_read_tab_set(base + 2, i, c64memrom_kernal64_read);
-        mem_read_tab_set(base + 3, i, c64memrom_kernal64_read);
-        mem_read_tab_set(base + 6, i, c64memrom_kernal64_read);
-        mem_read_tab_set(base + 7, i, c64memrom_kernal64_read);
-        mem_read_tab_set(base + 10, i, c64memrom_kernal64_read);
-        mem_read_tab_set(base + 11, i, c64memrom_kernal64_read);
-        mem_read_tab_set(base + 14, i, c64memrom_kernal64_read);
-        mem_read_tab_set(base + 15, i, c64memrom_kernal64_read);
-        mem_read_tab_set(base + 26, i, c64memrom_kernal64_read);
-        mem_read_tab_set(base + 27, i, c64memrom_kernal64_read);
-        mem_read_tab_set(base + 30, i, c64memrom_kernal64_read);
-        mem_read_tab_set(base + 31, i, c64memrom_kernal64_read);
-        mem_read_base_set(base + 2, i, c64memrom_kernal64_trap_rom - 0xe000);
-        mem_read_base_set(base + 3, i, c64memrom_kernal64_trap_rom - 0xe000);
-        mem_read_base_set(base + 6, i, c64memrom_kernal64_trap_rom - 0xe000);
-        mem_read_base_set(base + 7, i, c64memrom_kernal64_trap_rom - 0xe000);
-        mem_read_base_set(base + 10, i, c64memrom_kernal64_trap_rom - 0xe000);
-        mem_read_base_set(base + 11, i, c64memrom_kernal64_trap_rom - 0xe000);
-        mem_read_base_set(base + 14, i, c64memrom_kernal64_trap_rom - 0xe000);
-        mem_read_base_set(base + 15, i, c64memrom_kernal64_trap_rom - 0xe000);
-        mem_read_base_set(base + 26, i, c64memrom_kernal64_trap_rom - 0xe000);
-        mem_read_base_set(base + 27, i, c64memrom_kernal64_trap_rom - 0xe000);
-        mem_read_base_set(base + 30, i, c64memrom_kernal64_trap_rom - 0xe000);
-        mem_read_base_set(base + 31, i, c64memrom_kernal64_trap_rom - 0xe000);
+    if (board != 1) {
+        /* Setup Kernal ROM at $E000-$FFFF (memory configs 2, 3, 6, 7, 10,
+        11, 14, 15, 26, 27, 30, 31).  */
+        for (i = 0xe0; i <= 0xff; i++) {
+            mem_read_tab_set(base + 2, i, c64memrom_kernal64_read);
+            mem_read_tab_set(base + 3, i, c64memrom_kernal64_read);
+            mem_read_tab_set(base + 6, i, c64memrom_kernal64_read);
+            mem_read_tab_set(base + 7, i, c64memrom_kernal64_read);
+            mem_read_tab_set(base + 10, i, c64memrom_kernal64_read);
+            mem_read_tab_set(base + 11, i, c64memrom_kernal64_read);
+            mem_read_tab_set(base + 14, i, c64memrom_kernal64_read);
+            mem_read_tab_set(base + 15, i, c64memrom_kernal64_read);
+            mem_read_tab_set(base + 26, i, c64memrom_kernal64_read);
+            mem_read_tab_set(base + 27, i, c64memrom_kernal64_read);
+            mem_read_tab_set(base + 30, i, c64memrom_kernal64_read);
+            mem_read_tab_set(base + 31, i, c64memrom_kernal64_read);
+            mem_read_base_set(base + 2, i, c64memrom_kernal64_trap_rom - 0xe000);
+            mem_read_base_set(base + 3, i, c64memrom_kernal64_trap_rom - 0xe000);
+            mem_read_base_set(base + 6, i, c64memrom_kernal64_trap_rom - 0xe000);
+            mem_read_base_set(base + 7, i, c64memrom_kernal64_trap_rom - 0xe000);
+            mem_read_base_set(base + 10, i, c64memrom_kernal64_trap_rom - 0xe000);
+            mem_read_base_set(base + 11, i, c64memrom_kernal64_trap_rom - 0xe000);
+            mem_read_base_set(base + 14, i, c64memrom_kernal64_trap_rom - 0xe000);
+            mem_read_base_set(base + 15, i, c64memrom_kernal64_trap_rom - 0xe000);
+            mem_read_base_set(base + 26, i, c64memrom_kernal64_trap_rom - 0xe000);
+            mem_read_base_set(base + 27, i, c64memrom_kernal64_trap_rom - 0xe000);
+            mem_read_base_set(base + 30, i, c64memrom_kernal64_trap_rom - 0xe000);
+            mem_read_base_set(base + 31, i, c64memrom_kernal64_trap_rom - 0xe000);
+        }
     }
 
     /* Setup ROML at $8000-$9FFF.  */
@@ -262,6 +271,22 @@ void c64meminit(unsigned int base)
             }
         }
     }
+    /* Setup write Hook for when ROMH is NOT selected at $A000-$BFFF in 16K Game */
+    for (j = 24; j < 32; j++) {
+        if (!c64meminit_romh_config[j]) {
+            for (i = 0xa0; i <= 0xbf; i++) {
+                mem_set_write_hook(base + j, i, ramh_no_ultimax_store);
+            }
+        }
+    }
+    /* Setup write Hook for when ROMH is NOT selected at $a000-$bFFF when cart is off */
+    for (j = 0; j < 8; j++) {
+        if (!c64meminit_romh_config[j]) {
+            for (i = 0xa0; i <= 0xbf; i++) {
+                mem_set_write_hook(base + j, i, ramh_no_ultimax_store);
+            }
+        }
+    }
     /* Setup ROMH at $E000-$FFFF (ultimax)  */
     for (j = 16; j < 24; j++) {
         if (c64meminit_romh_config[j]) {
@@ -279,6 +304,13 @@ void c64meminit(unsigned int base)
 
     /* Setup Ultimax configuration.  */
     for (j = 16; j < 24; j++) {
+        if (board == 1) {
+            for (i = 0x08; i <= 0x0f; i++) {
+                mem_read_tab_set(base + j, i, ultimax_0800_0fff_read);
+                mem_set_write_hook(base + j, i, ultimax_0800_0fff_store);
+                mem_read_base_set(base + j, i, NULL);
+            }
+        }
         for (i = 0x10; i <= 0x7f; i++) {
             mem_read_tab_set(base + j, i, ultimax_1000_7fff_read);
             mem_set_write_hook(base + j, i, ultimax_1000_7fff_store);
