@@ -24,29 +24,29 @@
  *
  */
 
-#ifndef _ACTIONREPLAY_H
-#define _ACTIONREPLAY_H
+#ifndef VICE_ACTIONREPLAY_H
+#define VICE_ACTIONREPLAY_H
 
 #include <stdio.h>
 
 #include "types.h"
 
-extern BYTE REGPARM1 actionreplay_io1_read(WORD addr);
-extern void REGPARM2 actionreplay_io1_store(WORD addr, BYTE value);
-extern BYTE REGPARM1 actionreplay_io2_read(WORD addr);
-extern void REGPARM2 actionreplay_io2_store(WORD addr, BYTE value);
+uint8_t actionreplay_roml_read(uint16_t addr);
+void actionreplay_roml_store(uint16_t addr, uint8_t value);
 
-extern BYTE REGPARM1 actionreplay_roml_read(WORD addr);
-extern void REGPARM2 actionreplay_roml_store(WORD addr, BYTE value);
+void actionreplay_freeze(void);
 
-extern void actionreplay_freeze(void);
+void actionreplay_config_init(void);
+void actionreplay_reset(void);
+void actionreplay_config_setup(uint8_t *rawcart);
+int actionreplay_bin_attach(const char *filename, uint8_t *rawcart);
+int actionreplay_crt_attach(FILE *fd, uint8_t *rawcart);
+void actionreplay_detach(void);
+void actionreplay_powerup(void);
 
-extern void actionreplay_config_init(void);
-extern void actionreplay_reset(void);
-extern void actionreplay_config_setup(BYTE *rawcart);
-extern int actionreplay_bin_attach(const char *filename, BYTE *rawcart);
-extern int actionreplay_crt_attach(FILE *fd, BYTE *rawcart);
-extern void actionreplay_detach(void);
+struct snapshot_s;
+
+int actionreplay_snapshot_write_module(struct snapshot_s *s);
+int actionreplay_snapshot_read_module(struct snapshot_s *s);
 
 #endif
-
