@@ -33,11 +33,12 @@
 #define CARTRIDGE_INCLUDE_SLOTMAIN_API
 #include "c64cartsystem.h"
 #undef CARTRIDGE_INCLUDE_SLOTMAIN_API
-#include "c64export.h"
 #include "c64mem.h"
 #include "cartio.h"
 #include "cartridge.h"
 #include "diashowmaker.h"
+#include "export.h"
+#include "monitor.h"
 #include "snapshot.h"
 #include "types.h"
 #include "util.h"
@@ -74,52 +75,75 @@
 
 */
 
-#define DSM_CART_SIZE (8*0x400)
+#define DSM_CART_SIZE (8 * 0x400)
 
 /* ---------------------------------------------------------------------*/
 
-static BYTE dsm_io1_read(WORD addr)
+static int dsm_active = 0;
+
+/* We use the entire $de00-$deff range, even though the software only uses $de00,
+ * and we only react to address $de00, this is to fine tune the emulation when/if
+ * a working piece of software for the diashow maker shows up that uses an address
+ * other than $de00
+ */
+static uint8_t dsm_io1_read(uint16_t addr)
 {
     DBG(("io1 r %04x\n", addr));
     if (addr == 0) {
-        cart_config_changed_slotmain(2, 2, CMODE_READ);
+        cart_config_changed_slotmain(CMODE_RAM, CMODE_RAM, CMODE_READ);
         DBG(("Diashow Maker disabled\n"));
+        dsm_active = 0;
     }
     return 0; /* invalid */
 }
 
-static BYTE dsm_io1_peek(WORD addr)
+static uint8_t dsm_io1_peek(uint16_t addr)
 {
     return 0;
 }
 
-static void dsm_io1_store(WORD addr, BYTE value)
+/* We use the entire $de00-$deff range, even though the software only uses $de00,
+ * and we only react to address $de00, this is to fine tune the emulation when/if
+ * a working piece of software for the diashow maker shows up that uses an address
+ * other than $de00
+ */
+static void dsm_io1_store(uint16_t addr, uint8_t value)
 {
     DBG(("io1 w %04x %02x\n", addr, value));
     if (addr == 0) {
-        cart_config_changed_slotmain(2, 2, CMODE_READ);
+        cart_config_changed_slotmain(CMODE_RAM, CMODE_RAM, CMODE_READ);
         DBG(("Diashow Maker disabled\n"));
+        dsm_active = 0;
     }
 }
 
+static int dsm_dump(void)
+{
+    mon_out("ROM at $8000-$9FFF: %s\n", (dsm_active) ? "enabled" : "disabled");
+
+    return 0;
+}
+
 static io_source_t dsm_io1_device = {
-    CARTRIDGE_NAME_DIASHOW_MAKER,
-    IO_DETACH_CART,
-    NULL,
-    0xde00, 0xdeff, 0xff,
-    0, /* read is never valid */
-    dsm_io1_store,
-    dsm_io1_read,
-    dsm_io1_peek,
-    NULL,
-    CARTRIDGE_DIASHOW_MAKER,
-    0,
-    0
+    CARTRIDGE_NAME_DIASHOW_MAKER, /* name of the device */
+    IO_DETACH_CART,               /* use cartridge ID to detach the device when involved in a read-collision */
+    IO_DETACH_NO_RESOURCE,        /* does not use a resource for detach */
+    0xde00, 0xdeff, 0xff,         /* range for the device, reg: $de00, unknown mirrors:$de01-$deff */
+    0,                            /* read is never valid */
+    dsm_io1_store,                /* store function */
+    NULL,                         /* NO poke function */
+    dsm_io1_read,                 /* read function */
+    dsm_io1_peek,                 /* peek function */
+    dsm_dump,                     /* device state information dump function */
+    CARTRIDGE_DIASHOW_MAKER,      /* cartridge ID */
+    IO_PRIO_NORMAL,               /* normal priority, device read needs to be checked for collisions */
+    0,                            /* insertion order, gets filled in by the registration function */
+    IO_MIRROR_NONE                /* NO mirroring */
 };
 
 static io_source_list_t *dsm_io1_list_item = NULL;
 
-static const c64export_resource_t export_res = {
+static const export_resource_t export_res = {
     CARTRIDGE_NAME_DIASHOW_MAKER, 1, 1, &dsm_io1_device, NULL, CARTRIDGE_DIASHOW_MAKER
 };
 
@@ -128,26 +152,29 @@ static const c64export_resource_t export_res = {
 void dsm_freeze(void)
 {
     DBG(("Diashow Maker: freeze\n"));
-    cart_config_changed_slotmain(0, 0, CMODE_READ | CMODE_RELEASE_FREEZE);
+    cart_config_changed_slotmain(CMODE_8KGAME, CMODE_8KGAME, CMODE_READ | CMODE_RELEASE_FREEZE);
+    dsm_active = 1;
 }
 
 
 void dsm_config_init(void)
 {
-    cart_config_changed_slotmain(0, 0, CMODE_READ);
+    cart_config_changed_slotmain(CMODE_8KGAME, CMODE_8KGAME, CMODE_READ);
+    dsm_active = 1;
 }
 
-void dsm_config_setup(BYTE *rawcart)
+void dsm_config_setup(uint8_t *rawcart)
 {
     memcpy(roml_banks, rawcart, DSM_CART_SIZE);
-    cart_config_changed_slotmain(0, 0, CMODE_READ);
+    cart_config_changed_slotmain(CMODE_8KGAME, CMODE_8KGAME, CMODE_READ);
+    dsm_active = 1;
 }
 
 /* ---------------------------------------------------------------------*/
 
 static int dsm_common_attach(void)
 {
-    if (c64export_add(&export_res) < 0) {
+    if (export_add(&export_res) < 0) {
         return -1;
     }
 
@@ -155,7 +182,7 @@ static int dsm_common_attach(void)
     return 0;
 }
 
-int dsm_bin_attach(const char *filename, BYTE *rawcart)
+int dsm_bin_attach(const char *filename, uint8_t *rawcart)
 {
     DBG(("Diashow Maker: bin attach '%s'\n", filename));
     if (util_file_load(filename, rawcart, DSM_CART_SIZE, UTIL_FILE_LOAD_SKIP_ADDRESS) < 0) {
@@ -165,7 +192,7 @@ int dsm_bin_attach(const char *filename, BYTE *rawcart)
     return dsm_common_attach();
 }
 
-int dsm_crt_attach(FILE *fd, BYTE *rawcart)
+int dsm_crt_attach(FILE *fd, uint8_t *rawcart)
 {
     crt_chip_header_t chip;
 
@@ -186,59 +213,80 @@ int dsm_crt_attach(FILE *fd, BYTE *rawcart)
 
 void dsm_detach(void)
 {
-    c64export_remove(&export_res);
+    export_remove(&export_res);
     io_source_unregister(dsm_io1_list_item);
     dsm_io1_list_item = NULL;
 }
 
 /* ---------------------------------------------------------------------*/
 
-#define CART_DUMP_VER_MAJOR   0
-#define CART_DUMP_VER_MINOR   0
-#define SNAP_MODULE_NAME  "CARTDSM"
+/* CARTDSM snapshot module format:
+
+   type  | name   | version | description
+   --------------------------------------
+   BYTE  | active |   0.1   | cartridge active flag
+   ARRAY | ROML   |   0.0+  | 8192 BYTES of ROML data
+ */
+
+static const char snap_module_name[] = "CARTDSM";
+#define SNAP_MAJOR   0
+#define SNAP_MINOR   1
 
 int dsm_snapshot_write_module(snapshot_t *s)
 {
     snapshot_module_t *m;
 
-    m = snapshot_module_create(s, SNAP_MODULE_NAME,
-                          CART_DUMP_VER_MAJOR, CART_DUMP_VER_MINOR);
+    m = snapshot_module_create(s, snap_module_name, SNAP_MAJOR, SNAP_MINOR);
+
     if (m == NULL) {
         return -1;
     }
 
     if (0
+        || (SMW_B(m, (uint8_t)dsm_active) < 0)
         || (SMW_BA(m, roml_banks, DSM_CART_SIZE) < 0)) {
         snapshot_module_close(m);
         return -1;
     }
 
-    snapshot_module_close(m);
-    return 0;
+    return snapshot_module_close(m);
 }
 
 int dsm_snapshot_read_module(snapshot_t *s)
 {
-    BYTE vmajor, vminor;
+    uint8_t vmajor, vminor;
     snapshot_module_t *m;
 
-    m = snapshot_module_open(s, SNAP_MODULE_NAME, &vmajor, &vminor);
+    m = snapshot_module_open(s, snap_module_name, &vmajor, &vminor);
+
     if (m == NULL) {
         return -1;
     }
 
-    if ((vmajor != CART_DUMP_VER_MAJOR) || (vminor != CART_DUMP_VER_MINOR)) {
-        snapshot_module_close(m);
-        return -1;
+    /* Do not accept versions higher than current */
+    if (snapshot_version_is_bigger(vmajor, vminor, SNAP_MAJOR, SNAP_MINOR)) {
+        snapshot_set_error(SNAPSHOT_MODULE_HIGHER_VERSION);
+        goto fail;
     }
 
-    if (0
-        || (SMR_BA(m, roml_banks, DSM_CART_SIZE) < 0)) {
-        snapshot_module_close(m);
-        return -1;
+    /* new in 0.1 */
+    if (!snapshot_version_is_smaller(vmajor, vminor, 0, 1)) {
+        if (SMR_B_INT(m, &dsm_active) < 0) {
+            goto fail;
+        }
+    } else {
+        dsm_active = 0;
+    }
+
+    if (SMR_BA(m, roml_banks, DSM_CART_SIZE) < 0) {
+        goto fail;
     }
 
     snapshot_module_close(m);
 
     return dsm_common_attach();
+
+fail:
+    snapshot_module_close(m);
+    return -1;
 }
