@@ -34,42 +34,55 @@
 #ifndef VICE_MMC64_H
 #define VICE_MMC64_H
 
-#include "types.h"
+#include <stdio.h>
 
-struct export_s;
+#include "types.h"
+#include "c64cart.h"
+
+#define MMC64_REV_A  0
+#define MMC64_REV_B  1
+
+enum {
+    MMC64_TYPE_AUTO = 0,
+    MMC64_TYPE_MMC,
+    MMC64_TYPE_SD,
+    MMC64_TYPE_SDHC
+};
 
 /* FIXME: remove global clockport related variables */
 extern int mmc64_clockport_enabled;
 extern int mmc64_hw_clockport;
 
-extern int mmc64_cart_enabled(void);
-extern int mmc64_cart_active(void);
-extern void mmc64_config_init(struct export_s *export);
-extern int mmc64_roml_read(WORD addr, BYTE *byte);
-extern void mmc64_roml_store(WORD addr, BYTE byte);
-extern int mmc64_peek_mem(WORD addr, BYTE *value);
-extern void mmc64_passthrough_changed(struct export_s *export);
+int mmc64_cart_enabled(void);
+int mmc64_cart_active(void);
+void mmc64_config_init(export_t *export);
+int mmc64_roml_read(uint16_t addr, uint8_t *byte);
+void mmc64_roml_store(uint16_t addr, uint8_t byte);
+int mmc64_peek_mem(uint16_t addr, uint8_t *value);
+void mmc64_passthrough_changed(export_t *export);
 
-extern void mmc64_config_setup(BYTE *rawcart);
+void mmc64_config_setup(uint8_t *rawcart);
 
-extern int mmc64_crt_attach(FILE *fd, BYTE *rawcart);
-extern int mmc64_bin_attach(const char *filename, BYTE *rawcart);
-extern int mmc64_bin_save(const char *filename);
-extern int mmc64_crt_save(const char *filename);
-extern int mmc64_flush_image(void);
+int mmc64_crt_attach(FILE *fd, uint8_t *rawcart, const char *filename);
+int mmc64_bin_attach(const char *filename, uint8_t *rawcart);
+int mmc64_bin_save(const char *filename);
+int mmc64_crt_save(const char *filename);
+int mmc64_flush_image(void);
 
-extern int mmc64_resources_init(void);
-extern void mmc64_resources_shutdown(void);
-extern int mmc64_cmdline_options_init(void);
-extern void mmc64_init(void);
-extern void mmc64_detach(void);
-extern void mmc64_reset(void);
-extern int mmc64_enable(void);
-extern const char *mmc64_get_file_name(void);
-extern int mmc64_mmu_translate(unsigned int addr, BYTE **base, int *start, int *limit);
+int mmc64_resources_init(void);
+void mmc64_resources_shutdown(void);
+int mmc64_cmdline_options_init(void);
+void mmc64_init(void);
+void mmc64_detach(void);
+void mmc64_reset(void);
+int mmc64_enable(void);
+int mmc64_disable(void);
+const char *mmc64_get_file_name(void);
+int mmc64_mmu_translate(unsigned int addr, uint8_t **base, int *start, int *limit);
 
 struct snapshot_s;
-extern int mmc64_snapshot_read_module(struct snapshot_s *s);
-extern int mmc64_snapshot_write_module(struct snapshot_s *s);
+
+int mmc64_snapshot_read_module(struct snapshot_s *s);
+int mmc64_snapshot_write_module(struct snapshot_s *s);
 
 #endif
