@@ -27,14 +27,13 @@
 #ifndef VICE_IEC_C64EXP_H
 #define VICE_IEC_C64EXP_H
 
-struct drive_context_s;
+struct diskunit_context_s;
 
-extern int iec_c64exp_resources_init(void);
-extern void iec_c64exp_resources_shutdown(void);
-extern int iec_c64exp_cmdline_options_init(void);
-extern void iec_c64exp_init(struct drive_context_s *drv);
-extern void iec_c64exp_reset(struct drive_context_s *drv);
-extern void iec_c64exp_mem_init(struct drive_context_s *drv, unsigned int type);
+int iec_c64exp_resources_init(void);
+void iec_c64exp_resources_shutdown(void);
+int iec_c64exp_cmdline_options_init(void);
+void iec_c64exp_init(struct diskunit_context_s *drv);
+void iec_c64exp_reset(struct diskunit_context_s *drv);
+void iec_c64exp_mem_init(struct diskunit_context_s *drv, unsigned int type);
 
 #endif
-
