@@ -29,8 +29,7 @@
 
 struct snapshot_s;
 
-extern int plus4_snapshot_write_module(struct snapshot_s *s, int save_roms);
-extern int plus4_snapshot_read_module(struct snapshot_s *s);
- 
-#endif
+int plus4_snapshot_write_module(struct snapshot_s *s, int save_roms);
+int plus4_snapshot_read_module(struct snapshot_s *s);
 
+#endif
