@@ -42,7 +42,7 @@
 #include "palette.h"
 #include "types.h"
 
-/*#define DEBUG1520       1*/
+/*#define DEBUG1520       1 */
 /*#define DEBUG1520_A   1*/
 
 /*
@@ -62,6 +62,8 @@
 #define Y_PIXELS                (PIXELS_PER_STEP * (MAX_ROW+1))
 
 #define CR                      13
+#define SHIFT_CR                (CR + 128)
+#define LF                      10
 
 /*
  * Since the plotter's y coordinates follow the usual mathematic
@@ -69,18 +71,22 @@
  * rows from the top down, we'll invert the logical plotter coordinates
  * to obtain array indices (which start at 0 and increase as you go down
  * the page).
+ *
+ * In places, references are made to a ROM disassembly. It can be found at
+ * http://www.zimmers.net/anonftp/pub/cbm/firmware/printers/1520/1520-03.asm.html
  */
 
 enum command_stage {
     start,
     letter_seen,
     x_seen,
-    y_seen
+    y_seen,
+    skip_rest
 };
 
 struct plot_s {
     int prnr;
-    BYTE (*sheet)[Y_PIXELS][X_PIXELS];
+    uint8_t (*sheet)[Y_PIXELS][X_PIXELS];
     int colour;         /* sa = 2 */
     int colour_accu;
     int charsize;       /* sa = 3; segment multiplication */
@@ -98,10 +104,10 @@ struct plot_s {
     int command_x;
     int command_y;
     int command_flags;
-    int abs_origin_x, abs_origin_y;     /* relative to start of page */
-    int rel_origin_x, rel_origin_y;     /* relative to abs_origin */
-    int cur_x, cur_y;                   /* relative to abs_origin */
-    int lowest_y;                       /* relative to start of page */
+    int abs_origin_x, abs_origin_y;     /* steps relative to start of page */
+    int rel_origin_x, rel_origin_y;     /* steps relative to abs_origin */
+    int cur_x, cur_y;                   /* steps relative to abs_origin */
+    int lowest_y;                       /* steps relative to start of page */
 };
 typedef struct plot_s plot_t;
 
@@ -109,19 +115,21 @@ static plot_t drv_1520[NUM_OUTPUT_SELECT];
 static palette_t *palette = NULL;
 
 /* Logging goes here.  */
-static log_t drv1520_log = LOG_ERR;
+static log_t drv1520_log = LOG_DEFAULT;
 
 #define PIXEL_INDEX_WHITE       0
 #define PIXEL_INDEX_BLACK       1
 
-static BYTE tochar[] = {
+static const uint8_t tochar[] = {
     OUTPUT_PIXEL_WHITE,         /* paper */
     OUTPUT_PIXEL_BLACK,         /* 1520 colour numbers: 0 - 3 */
-    OUTPUT_PIXEL_BLUE, 
-    OUTPUT_PIXEL_GREEN, 
-    OUTPUT_PIXEL_RED, 
+    OUTPUT_PIXEL_BLUE,
+    OUTPUT_PIXEL_GREEN,
+    OUTPUT_PIXEL_RED,
 };
 
+static void reset_scribe_state(plot_t *mps);
+
 /* ------------------------------------------------------------------------- */
 /* 1520 plotter engine. */
 
@@ -140,19 +148,19 @@ static void write_lines(plot_t *mps, int lines)
         for (x = 0; x < X_PIXELS; x++) {
             output_select_putc(prnr, tochar[(*mps->sheet)[y][x]]);
         }
-        output_select_putc(prnr, (BYTE)OUTPUT_NEWLINE);
+        output_select_putc(prnr, (uint8_t)OUTPUT_NEWLINE);
     }
 }
 
 /*
- * The current location becomes the new hard origin.
+ * The current location becomes the new absolute origin.
  * Anything that is now too far away in the "up" direction
  * can be printed for sure and erased from our buffered sheet.
  */
-static void reset_hard_origin(plot_t *mps)
+static void reset_abs_origin(plot_t *mps)
 {
 #if DEBUG1520
-    log_message(drv1520_log, "reset_hard_origin: abs_origin_y=%d lowest_y=%d cur_y=%d", mps->abs_origin_y, mps->lowest_y, mps->cur_y);
+    log_message(drv1520_log, "reset_abs_origin: abs_origin_y=%d lowest_y=%d cur_y=%d", mps->abs_origin_y, mps->lowest_y, mps->cur_y);
 #endif
     mps->abs_origin_x += mps->cur_x;
     mps->abs_origin_y += mps->cur_y;
@@ -161,21 +169,22 @@ static void reset_hard_origin(plot_t *mps)
 #endif
 
     mps->cur_x = mps->cur_y = 0;
+    reset_scribe_state(mps);
 
-    /* If the hard origin is reset, does the relative origin too reset?
+    /* If the absolute origin is reset, does the relative origin too reset?
      * Assume "yes" for now.
      */
     mps->rel_origin_x = mps->rel_origin_y = 0;
 
     if (mps->abs_origin_y < -MAX_Y_COORD) {
-        int y_segments = -mps->abs_origin_y - MAX_Y_COORD;
-        int y_pixels = y_segments * PIXELS_PER_STEP;
+        int y_steps = -mps->abs_origin_y - MAX_Y_COORD;
+        int y_pixels = y_steps * PIXELS_PER_STEP;
         int remaining_y_pixels = Y_PIXELS - y_pixels;
 #if DEBUG1520
-        log_message(drv1520_log, "reset_hard_origin: output and shift %d pixels (%d Y-coordinates)", y_pixels, y_segments);
+        log_message(drv1520_log, "reset_abs_origin: output and shift %d pixels (%d Y-coordinates)", y_pixels, y_steps);
 #endif
 
-        write_lines(mps, y_segments);
+        write_lines(mps, y_steps);
         memmove(&(*mps->sheet)[0][0],                   /* destination */
                 &(*mps->sheet)[y_pixels][0],            /* source */
                 remaining_y_pixels * X_PIXELS);
@@ -184,14 +193,23 @@ static void reset_hard_origin(plot_t *mps)
                 PIXEL_INDEX_WHITE,
                 y_pixels * X_PIXELS);
 
-        mps->abs_origin_y += y_segments;
-        mps->lowest_y += y_segments;
+        mps->abs_origin_y += y_steps;
+        mps->lowest_y += y_steps;
 #if DEBUG1520
-    log_message(drv1520_log, "                 : abs_origin_y=%d lowest_y=%d", mps->abs_origin_y, mps->lowest_y);
+        log_message(drv1520_log, "                 : abs_origin_y=%d lowest_y=%d", mps->abs_origin_y, mps->lowest_y);
 #endif
     }
 }
 
+/*
+ * Set the relative origin to the place where we are now.
+ */
+static void set_rel_origin(plot_t *mps)
+{
+    mps->rel_origin_x = mps->cur_x;
+    mps->rel_origin_y = mps->cur_y;
+}
+
 static void eject(plot_t *mps)
 {
 #if DEBUG1520
@@ -208,17 +226,36 @@ static void eject(plot_t *mps)
     mps->lowest_y = mps->abs_origin_y;
 }
 
+/*
+ * Originally: every line draw re-starts the dashed line state.
+ *
+ * Now: the rom disassembly suggests that the scribe state gets reset on
+ * every move or draw (pen up or down) or when setting the absolute
+ * origin. Of course you see the effect only when the pen is down.
+ * Therefore we can leave out the calls where there is only movement.
+ * See 0e13  sta dash_counter
+ *
+ * The difference is what happens with lines drawn for letters.
+ */
+static void reset_scribe_state(plot_t *mps)
+{
+    mps->scribe_state = 0;
+}
+
 static inline int vice_min(int a, int b)
 {
     return a < b ? a : b;
 }
 
+/* Used in DEBUG code only */
+#if 0
 static inline int vice_max(int a, int b)
 {
     return a > b ? a : b;
 }
+#endif
 
-static void mix(BYTE *old, int new)
+static void mix(uint8_t *old, int new)
 {
     if (*old == PIXEL_INDEX_WHITE) {
         *old = new;
@@ -235,10 +272,17 @@ static void plot(plot_t *mps, int x, int y)
 #if DEBUG1520_A
     log_message(drv1520_log, "plot: (%4d,%4d) scribe=%d state=%d", x, y, mps->scribe, mps->scribe_state);
 #endif
+    if (x >= X_PIXELS || x < 0 || y > Y_PIXELS || y < 0) {
+#if DEBUG1520
+        log_message(drv1520_log, "plot: (%4d,%4d) out of bounds", x, y);
+#endif
+        return;
+    }
+
     if (mps->scribe) {
         if (mps->scribe_state < mps->scribe) {
             mix(&(*mps->sheet)[y][x], PIXEL_INDEX_BLACK + mps->colour);
-        } 
+        }
         mps->scribe_state++;
         if (mps->scribe_state >= 2 * mps->scribe) {
             mps->scribe_state = 0;
@@ -318,7 +362,7 @@ static void bresenham_par(plot_t *mps, int x0, int y0, int x1, int y1, int wd)
         if (e2 > -dy) {
             err -= dy;
             xo  += sx;
-            /* mps->colour = vice_max(1, (mps->colour+1) % 4); // DEBUG */
+            /* mps->colour = vice_max(1, (mps->colour+1) % 4); */ /* DEBUG */
             mps->scribe_state = scribe;
             bresenham(mps, x0 + xo, y0 + yo, x1 + xo, y1 + yo);
             if (wd > 1) {
@@ -329,7 +373,7 @@ static void bresenham_par(plot_t *mps, int x0, int y0, int x1, int y1, int wd)
         if (e2 < dx) {
             err += dx;
             yo  += sy;
-            /* mps->colour = vice_max(1, (mps->colour+1) % 4); // DEBUG */
+            /* mps->colour = vice_max(1, (mps->colour+1) % 4); */ /* DEBUG */
             mps->scribe_state = scribe;
             bresenham(mps, x0 + xo, y0 + yo, x1 + xo, y1 + yo);
             if (wd > 1) {
@@ -343,7 +387,6 @@ static void bresenham_par(plot_t *mps, int x0, int y0, int x1, int y1, int wd)
 #define Assert(val, expr) do {  \
         if (!(expr)) {          \
             log_error(drv1520_log, "%s %d: assertion %s failed: %d", __FILE__, __LINE__, #expr, val);\
-            return;             \
         }                       \
     } while (0)
 
@@ -391,122 +434,119 @@ static void draw(plot_t *mps, int from_x, int from_y, int to_x, int to_y)
       to_x *= PIXELS_PER_STEP;
       to_y *= PIXELS_PER_STEP;
 
-    /* Every line re-starts the dashed line state */
-    mps->scribe_state = 0;
-
-    //bresenham(mps, from_x, from_y, to_x, to_y);
+    /*bresenham(mps, from_x, from_y, to_x, to_y);*/
     bresenham_par(mps, from_x, from_y, to_x, to_y, PIXELS_PER_STEP);
 }
 
 /*
  * Reverse-engineered drawing commands for the characters.
  */
-char *punct[32] = {
+static const char * const punct[32] = {
     /*   */ NULL,
-    /* ! */ "66 d 8624 u 88 d 888886222224 u",
-    /* " */ "988888 d 2 u 96 d 2",
-    /* # */ "9 d 8888 u 1 d 6666 u 7 d 2222 u 9 d 4444",
-    /* $ */ "96 d 6697447966 u 7 d 222222",
-    /* % */ "8 d 9999 u 444 d 4268 u 332 d 6248",
-    /* & */ "6666 d 777789321123699",               // long line first
-    /* ' */ "888899 d 22",
-    /* ( */ "6666 d 478888896",
-    /* ) */ "66   d 698888874",
-    /* * */ "8 d 9999 u 4444 d 3333 u 7788 d 2222", // vertical down last
-    /* + */ "888 d 6666 u 11 d 8888",               // right, then down
+    /* ! */ "96 d 8624 u 88 d 8888622224",
+    /* " */ "9888888 d 2 u 96 d 2",
+    /* # */ "98 d 8888 u 66 d 2222 u 9 d 4444 u 88 d 6666",
+    /* $ */ "98 d 6697447966 u 7 d 222222",
+    /* % */ "88 d 9999 u 444 d 4268 u 332 d 6248",
+    /* & */ "9666 d 777789321123699",                /* long line first */
+    /* ' */ "88899 d 88",
+    /* ( */ "966 d 47888896",
+    /* ) */ "96  d 69888874",
+    /* * */ "88 d 9999 u 4444 d 3333 u 7788 d 2222", /* vertical down last */
+    /* + */ "99 d 8888 u 11 d 6666",
     /* , */ "66 d 98426",
-    /* - */ "888 d 66666",
-    /* . */ "666 d 8624",
-    /* / */ "d 9999",
-    /* 0 */ "8 d 8888966322221447 9999",
-    /* 1 */ "88888 d 9222222 u 4 d 66",             // down
-    /* 2 */ "88888 d 9663211116666",
-    /* 3 */ "88888 d 9663214 u 6 d321447",
-    /* 4 */ "9966 d 44448999222222",
-    /* 5 */ "8 d 3669887444886666",
-    /* 6 */ "888 d 966322144788889663",
-    /* 7 */ "d 9999884444",
-    /* 8 */ "999 d 98744123 u 66 d 3214478966",
-    /* 9 */ "8 d 36698888744123669",
-    /* : */ "99 d 8624 u 88 d 8624",
-    /* ; */ "96 d 98426 u 88 d 8426",
-    /* < */ "999988 d 111333",
-    /* = */ "88 d 6666 u 7744 d 6666",
-    /* > */ "d 999777",
-    /* ? */ "88888 d 96632142 u 2 d 2"
+    /* - */ "8888 d 6666",
+    /* . */ "96 d 8624",
+    /* / */ "8 d 99999",
+    /* 0 */ "88 d 9999 74412222 36698888",
+    /* 1 */ "988888 d 9222222 u 4 d 66",             /* down */
+    /* 2 */ "888888 d 9663211116666",
+    /* 3 */ "888888 d 9663214 u 6 d 321447",
+    /* 4 */ "9996 d 44448999222222",
+    /* 5 */ "88 d 3669887444886666",
+    /* 6 */ "8888 d 966322144788889663",
+    /* 7 */ "8 d 9999884444",
+    /* 8 */ "8889 d 789663213214478966",
+    /* 9 */ "88 d 36698888744123669",
+    /* : */ "89 d 8624 u 888 d 8624",
+    /* ; */ "9 d 98426 u 888 d 8426",
+    /* < */ "9666 d 777999",
+    /* = */ "988 d 666 u 774 d 666",
+    /* > */ "8 d 999777",
+    /* ? */ "888888 d 96632142 u 2 d 2"
 };
-/* typically 4 units wide, and 6 units high. */
-char *uppercase[32] = {
-    /* @ */ "669 d 8884422266 98874412223666", /* anti-clockwise from inside */
-    /* A */ "d 888899332222 u 888 d 4444",
-    /* B */ "d 888888666321444 u 666 d 321444",
-    /* C */ "99998 d 744122223669",
-    /* D */ "d 888888666322221444",
-    /* E */ "6666 d 4444 888888 6666 u 1114 d 666 u 1114",
-    /* F */ "d 8888886666 u 1114 d 666",
-    /* G */ "99998 d 74412222366688844",
-    /* H */ "d 888888 u 222 d 6666 u 888 d 222222",
-    /* I */ "6 d 66 u 4 d 888888 u 4 d 66",
-    /* J */ "8 d 36988888",
-    /* K */ "d 888888 u 6666 d 1111 u 9 d 333",
-    /* L */ "888888 d 222222 6666",
-    /* M */ "d 888888332 u 8 d 99222222",
-    /* N */ "d 888888 u 2 d 3333 u 2 d 888888",
-    /* O */ "8 d 8888966322221447",
-    /* P */ "d 888888666321444",
-    /* Q */ "8 d 8888966322221447 u 96 d 33",
-    /* R */ "d 888888666321444 u 6 d 333",
-    /* S */ "8 d 36698744789663",
-    /* T */ "66 d 888888 u 44 d 6666",
-    /* U */ "888888 d 22222366988888",
-    /* V */ "888888 d 222233998888",
-    /* W */ "888888 d 222222998 u 2 d 33888888",
-    /* X */ "d 899998 u 4444 d 233332 ",
-    /* Y */ "66 d 888778 u 6666 d 211", /* ?? */
-    /* Z */ "888888 d 66662111126666",
-    /* [ */ "66 d 44 888888 66",
-    /*  */ "99999 d 41222266 u 887 d 44 u 1 d 1369 u 4 d 7",   /* ?? */
-    /* ] */ "d 666 888888 44 ",
-    /* ^ */ "66 d 88888 11 u 99 d 33",
-    /* <-*/ "66999 d 44444 333 u 777 d 999",
+/* typically 4 units wide, and 7 units high (baseline 1). */
+static const char * const uppercase[32] = {
+    /* @ */ "999 d 884422266 98874412223666", /* anti-clockwise from inside */
+    /* A */ "8 d 888899332222 u 888 d 4444",
+    /* B */ "8 d 888888666321444 u 666 d 321444",
+    /* C */ "9966 d 144788889663",
+    /* D */ "8 d 888888666322221444",
+    /* E */ "9666 d 4444 888888 6666 u 122 d 444",
+    /* F */ "8 d 8888886666 u 1114 d 666",
+    /* G */ "999988 d 74412222366688844",
+    /* H */ "8 d 888888 u 6666 d 222222 u 7774 d 6666",
+    /* I */ "9 d 66 u 4 d 888888 u 4 d 66",
+    /* J */ "88 d 36988888",
+    /* K */ "8 d 888888 u 6666 d 1111 u 9 d 333",
+    /* L */ "8888888 d 222222 6666",
+    /* M */ "8 d 88888833 28 99222222",
+    /* N */ "8 d 888888 u 2 d 3333 u 88888 d 222222",
+    /* O */ "88 d 8888966322221447",
+    /* P */ "8 d 888888666321444",
+    /* Q */ "998 d 33 u 4 d 4478888966322221",
+    /* R */ "8 d 888888666321444 u 6 d 333",
+    /* S */ "88 d 36698744789663",
+    /* T */ "96 d 888888 u 44 d 6666",
+    /* U */ "8888888 d 22222366988888",
+    /* V */ "8888888 d 222233998888",
+    /* W */ "8888888 d 22222299 u 8 d 233888888",
+    /* X */ "8 d 899998 u 4444 d 233332 ",
+    /* Y */ "69 d 888998 u 4444 d 233",
+    /* Z */ "8888888 d 66662111126666",
+    /* [ */ "69 d 44 888888 66",
+/* pound */ "999998 d 4122214793366 u 777 d 66",
+    /* ] */ "9 d 66 888888 44 ",
+    /* ^ */ "69 d 88888 11 u 99 d 33",
+    /* <-*/ "99 d 7799 u 11 d 66666",
 };
-/* Typically 3 units wide and 4 high (plus ascenders / descenders). */
-char *lowercase[32] = {
-    /* --*/ "888 d 666666",
-    /* a */ "8888 d 66322147896323",
-    /* b */ "888888 d 22222266988744",
-    /* c */ "999 d 74122369",
-    /* d */ "999888 d 22222244788966",
-    /* e */ "88 d 666874122366",
-    /* f */ "6 d 888886 u 112 d 66",// ??
-    /* g */ "996 d 1478963222147",
-    /* h */ "d 888888 u 22 d 663222",
-    /* i */ "66 d 888 u 8 d 8",
-    /* j */ "d 369888 u 8 d 8",
-    /* k */ "d 888888 u 336 d 111 u 9 d 33",
-    /* l */ "888889 d 222222 6",
-    /* m */ "d 8888 u 2 d 93222 u 888 d 93222",
-    /* n */ "d 8888 u 2 d 963222",
-    /* o */ "8 d 8896322147",           // clockwise
-    /* p */ "8 d 66987442222",          // anti-clockwise
-    /* q */ "6663 d 4888884412366",     // anti-clockwise
-    /* r */ "8888 d 3222 u 888 d 963",  // from serif down, then top bow
-    /* s */ "8 d 369747963",
-    /* t */ "8888 d 66 u 78 d 2222226", // down
-    /* u */ "8888 d 22226668888",       // down right up
-    /* v */ "8888 d 22339988",
-    /* w */ "8888 d 222398 u 2 d 39888",
-    /* x */ "d 9999 u 4444 d 3333",
-    /* y */ "8888 d 233 u 998 d 21111", // top left to middle, top right to left bottom
-    /* z */ "8888 d 666611116666",      // top to bottom
-    /* | */ "9998888 d 22222222",
-    /* __*/ "2 d 66666666",
-    /* ^_*/ "d 666666777111",// too wide?
-    /* pi*/ "6 d 88866222 u 44488 d 9669 ",// clockwise square, then hat
-    /* []*/ "d 8888886666622222244444",
+/* Typically 3 units wide and 5 high (plus ascenders / descenders). */
+static const char * const lowercase[32] = {
+    /* --*/ "8888 d 6666666",
+    /* a */ "996 d 1478963 u 774 d 663223",
+    /* b */ "8888888 d 22222266988744",
+    /* c */ "996 d 14788963",
+    /* d */ "99988 d 44122366888888",
+    /* e */ "888 d 666874122366",
+    /* f */ "96 d 8888886 u 112 d 66",
+    /* g */ "8 d 3698887412369",
+    /* h */ "8 d 888888 u 22 d 663222",
+    /* i */ "69 d 888 u 8 d 8",
+    /* j */ "8 d 369888 u 8 d 8",
+    /* k */ "669 d 77 u 7888 d 222222 u 8 d 999",
+    /* l */ "8888889 d 222222 6",
+    /* m */ "8 d 8888 u 2 d 93222 u 888 d 93222",
+    /* n */ "8 d 8888 2 963222",
+    /* o */ "88 d 8896322147",           /* clockwise */
+    /* p */ "88 d 669874422222",         /* anti-clockwise */
+    /* q */ "6666 d 4888884412366",      /* anti-clockwise */
+    /* r */ "88888 d 3222 u 888 d 96",   /* from serif down, then top bow */
+    /* s */ "88 d 369747963",
+    /* t */ "88888 d 66 u 78 d 2222226", /* down */
+    /* u */ "88888 d 22226668888",       /* down right up */
+    /* v */ "88888 d 22339988",
+    /* w */ "88888 d 22239 82 39888",
+    /* x */ "8 d 9999 u 4444 d 3333",
+    /* y */ "88888 d 233 u 998 d 21111", /* top left to middle, top right to left bottom */
+    /* z */ "88888 d 666611116666",      /* top to bottom */
+    /* | */ "666 d 8888888",
+    /* __*/ "d 6666666",
+    /* ^_*/ "8 d 999333 444444",
+    /* pi*/ "888 d 966222 u 44 d 888669",
+    /* []*/ "8 d 8888886666622222244444",
 };
 
-int dir[20] = { /* delta-x, delta-y */
+static const int dir[20] = { /* delta-x, delta-y */
      0,  0,     /* 0 */
     -1, -1,     /* 1   7 8 9 */
      0, -1,     /* 2   4 5 6 */
@@ -519,18 +559,22 @@ int dir[20] = { /* delta-x, delta-y */
      1,  1,     /* 9 */
 };
 
-#define LINEFEED       10
+#define LINE_HEIGHT    10       /* line height */
 #define LW              4       /* letter width */
 #define SW              6       /* letter pitch */
-#define LH              6       /* letter height */
+#define LH              7       /* letter height */
 
-void draw_char(plot_t *mps, char *commands)
+static void draw_char(plot_t *mps, const char *commands)
 {
     int x = mps->cur_x;
     int y = mps->cur_y;
     int s = mps->charsize;
     int pen_down = 0;
     char command;
+    char prev_command = '?';
+
+    /* Set relative origin; see disasm 0d71: char_found  jsr set_relorigin */
+    set_rel_origin(mps);
 
     /*
      * For normal and rotated letters, the tops of the
@@ -539,7 +583,6 @@ void draw_char(plot_t *mps, char *commands)
      */
     if (mps->rotation) {
         y += LH * s;
-        x += s;
     }
 
     while ((command = *commands++) != '\0') {
@@ -561,27 +604,89 @@ void draw_char(plot_t *mps, char *commands)
 
             /* TODO: this is probably not how it was really handled: */
             newx = vice_min(MAX_COL, newx);
+
             if (pen_down) {
+                /*
+                 * The ROM code draws longer lines in one go, but we do it in
+                 * unit square segments. Reset the scribe state only on a
+                 * really new line.
+                 */
+                if (prev_command != command) {
+                    reset_scribe_state(mps);
+                }
+
                 draw(mps, x, y, newx, newy);
             }
 
             x = newx;
             y = newy;
+            prev_command = command;
         }
     }
 }
 
-static void print_char_text(plot_t *mps, BYTE c)
+/*
+ * Move the pen all the way back to the left.
+ */
+static void move_left_side(plot_t *mps)
+{
+    mps->cur_x = 0;
+}
+
+/*
+ * Move the paper upward by one line.
+ * This sets the absolute origin to the new position.
+ */
+static void line_feed(plot_t *mps)
+{
+    mps->cur_y -= mps->charsize * LINE_HEIGHT;
+    reset_abs_origin(mps);
+}
+
+/*
+ * Move to the next line without resetting quote mode.
+ */
+static void carriage_return(plot_t *mps)
+{
+    move_left_side(mps);
+    line_feed(mps);
+}
+
+/*
+ * Move to the next line and reset quote mode.
+ */
+static void print_char_cr(plot_t *mps)
+{
+    carriage_return(mps);
+    mps->quote_mode = 0;
+}
+
+/*
+ * On the computer shift-CR would reset quote mode, like CR, but
+ * the 1520 apparently doesn't do it like that.
+ * See disassembly 0c7e do_shcr: jsr move_left_side; beq nextchar
+ */
+static void print_char_shcr(plot_t *mps)
+{
+    move_left_side(mps);
+}
+
+static void print_char_text(plot_t *mps, uint8_t c)
 {
     int underline = 0;
-    char **tab;
+    const char * const *tab;
 
-    if (c == CR) {
-        mps->cur_x = 0;
-        mps->cur_y -= mps->charsize * LINEFEED;
-        reset_hard_origin(mps);
-        mps->quote_mode = 0;
+    if (c == CR || c == LF) {
+        print_char_cr(mps);
         return;
+    } else if (c == SHIFT_CR) {
+        print_char_shcr(mps);
+        return;
+    }
+
+    /* Wrap long lines, without resetting quote mode */
+    if (mps->cur_x >= MAX_COL) {
+        carriage_return(mps);
     }
 
     if (c == 0xFF) {    /* Handle pi */
@@ -625,12 +730,16 @@ static void print_char_text(plot_t *mps, BYTE c)
         draw_char(mps, tab[c & 0x1F]);
     }
 
+    if (underline) {
+        /*
+         * TODO: may also set absolute origin;
+         * see disassembly 0c67: jsr set_absorigin; ldx #font.LOW_LINE
+         */
+        draw_char(mps, "d 6666666");
+    }
+
     mps->cur_x += mps->charsize * SW;
 
-    if (underline) {
-        draw(mps, mps->cur_x - mps->charsize * SW, mps->cur_y - 1,
-                  mps->cur_x                     , mps->cur_y - 1);
-    }
 }
 
 #define INCOMPLETE      99999
@@ -643,7 +752,7 @@ static void print_char_text(plot_t *mps, BYTE c)
  * White space is ignored.
  * Other characters reset the accumulator to 0.
  */
-static int numparser(int *accu, BYTE c)
+static int numparser(int *accu, uint8_t c)
 {
     /*
      * TODO: what does it do with other characters?
@@ -688,7 +797,7 @@ static int numparser(int *accu, BYTE c)
  * therefore the number overflows rather a lot.
  * (the 1520 indeed interprets 240E0 and 24E1 etc as 0!)
  */
-static int fnumsparser(int *accu, int *state, BYTE c)
+static int fnumsparser(int *accu, int *state, uint8_t c)
 {
     /*
      * TODO: what does it do with other characters?
@@ -715,6 +824,11 @@ static int fnumsparser(int *accu, int *state, BYTE c)
         }
     } else {
         if (c >= '0' && c <= '9') {
+            /*
+             * TODO: if too many digits, ignore the remainder.
+             * See disassembly 0cc9 sta count, 0d0a beq digits_finished,
+             * digits_finished 0d25
+             */
             if (!(*state & (FNUM_DOT|FNUM_E))) {
                 *accu *= 10;
                 *accu += (c - '0');
@@ -748,13 +862,10 @@ done:
     return INCOMPLETE;
 }
 
-static void print_char_plot(plot_t *mps, const BYTE c)
+static void print_char_plot(plot_t *mps, const uint8_t c)
 {
     int value;
 
-    /*
-     * TODO: what does it do with incorrect command letters?
-     */
     switch (mps->command_stage) {
     case start:
         if (strchr("HIMDRJ", c)) {
@@ -766,6 +877,9 @@ static void print_char_plot(plot_t *mps, const BYTE c)
 #if DEBUG1520
             log_message(drv1520_log, "print_char_plot: command_stage := letter_seen");
 #endif
+        } else {
+            mps->command = '?';
+            mps->command_stage = skip_rest;
         }
         break;
     case letter_seen:
@@ -787,6 +901,7 @@ static void print_char_plot(plot_t *mps, const BYTE c)
         }
         break;
     case y_seen:
+    case skip_rest:
         /* By now we really hope to see a CR - ignore anything else. */
         break;
     }
@@ -799,7 +914,7 @@ static void print_char_plot(plot_t *mps, const BYTE c)
 
         /* TODO: Q: what if not enough numbers are given?
          * A: It seems that 0 is used.
-         * 
+         *
          * TODO: Q: what if the numbers are too large? >= 999
          * A1: x > 480: head tries to move there anyway; motor buzzes.
          */
@@ -811,8 +926,7 @@ static void print_char_plot(plot_t *mps, const BYTE c)
             mps->cur_y = 0;
             break;
         case 'I':       /* set relative origin here */
-            mps->rel_origin_x = mps->cur_x;
-            mps->rel_origin_y = mps->cur_y;
+            set_rel_origin(mps);
             break;
         case 'M':       /* move to (x,y) relative to absolute origin */
             mps->cur_x = mps->command_x;
@@ -825,6 +939,7 @@ static void print_char_plot(plot_t *mps, const BYTE c)
         case 'D':       /* draw to (x,y) relative to absolute origin */
             new_x = mps->command_x;
             new_y = mps->command_y;
+            reset_scribe_state(mps);
             draw(mps, mps->cur_x, mps->cur_y, new_x, new_y);
             mps->cur_x = new_x;
             mps->cur_y = new_y;
@@ -832,6 +947,7 @@ static void print_char_plot(plot_t *mps, const BYTE c)
         case 'J':       /* draw to (x,y) relative to relative origin */
             new_x = mps->rel_origin_x + mps->command_x;
             new_y = mps->rel_origin_y + mps->command_y;
+            reset_scribe_state(mps);
             draw(mps, mps->cur_x, mps->cur_y, new_x, new_y);
             mps->cur_x = new_x;
             mps->cur_y = new_y;
@@ -848,7 +964,7 @@ static void print_char_plot(plot_t *mps, const BYTE c)
     }
 }
 
-static void print_char_select_colour(plot_t *mps, const BYTE c)
+static void print_char_select_colour(plot_t *mps, const uint8_t c)
 {
     int value;
 
@@ -859,7 +975,7 @@ static void print_char_select_colour(plot_t *mps, const BYTE c)
     }
 }
 
-static void print_char_select_character_size(plot_t *mps, const BYTE c)
+static void print_char_select_character_size(plot_t *mps, const uint8_t c)
 {
     int value;
 
@@ -870,7 +986,7 @@ static void print_char_select_character_size(plot_t *mps, const BYTE c)
     }
 }
 
-static void print_char_select_character_rotation(plot_t *mps, const BYTE c)
+static void print_char_select_character_rotation(plot_t *mps, const uint8_t c)
 {
     int value;
 
@@ -881,7 +997,7 @@ static void print_char_select_character_rotation(plot_t *mps, const BYTE c)
     }
 }
 
-static void print_char_select_scribe(plot_t *mps, const BYTE c)
+static void print_char_select_scribe(plot_t *mps, const uint8_t c)
 {
     int value;
 
@@ -899,7 +1015,7 @@ static void print_char_select_scribe(plot_t *mps, const BYTE c)
     }
 }
 
-static void print_char_select_lowercase(plot_t *mps, const BYTE c)
+static void print_char_select_lowercase(plot_t *mps, const uint8_t c)
 {
     int value;
 
@@ -933,7 +1049,7 @@ static void draw_four_squares(plot_t *mps)
     }
     mps->cur_x = 0;
     mps->cur_y -= 4 * STEPS_PER_MM;
-    reset_hard_origin(mps);
+    reset_abs_origin(mps);
     mps->colour = 0;
 }
 
@@ -950,16 +1066,17 @@ static void power_on_reset(plot_t *mps)
 
     mps->prnr = prnr;
     mps->charsize = 1 << 1;
-    mps->sheet = lib_calloc(Y_PIXELS, X_PIXELS * sizeof(BYTE));
+    mps->sheet = lib_calloc(Y_PIXELS, X_PIXELS * sizeof(uint8_t));
     mps->abs_origin_x = 0;
     mps->abs_origin_y = -VERTICAL_CLEARANCE;
 
     draw_four_squares(mps);
 }
 
-static void print_char_reset(plot_t *mps, const BYTE c)
+static void print_char_reset(plot_t *mps, const uint8_t c)
 {
     if (c == CR) {
+        eject(mps);     /* don't lose plotted data */
         power_on_reset(mps);
     }
 }
@@ -970,7 +1087,7 @@ static void print_char_reset(plot_t *mps, const BYTE c)
 static int drv_1520_open(unsigned int prnr, unsigned int secondary)
 {
 #if DEBUG1520
-    log_message(drv1520_log, "drv_1520_open: sa=%d prnr=%d", secondary, prnr);
+    log_message(drv1520_log, "drv_1520_open: sa=%u prnr=%u", secondary, prnr);
 #endif
 
     /* Is this the first open? */
@@ -978,8 +1095,8 @@ static int drv_1520_open(unsigned int prnr, unsigned int secondary)
         output_parameter_t output_parameter;
         plot_t *mps = &drv_1520[prnr];
 
-        output_parameter.maxcol = X_PIXELS;
-        output_parameter.maxrow = Y_PIXELS;
+        output_parameter.maxcol = X_PIXELS+1;
+        output_parameter.maxrow = Y_PIXELS+1;
         output_parameter.dpi_x = (PIXELS_PER_STEP * STEPS_PER_MM * 254) / 10;
         output_parameter.dpi_y = (PIXELS_PER_STEP * STEPS_PER_MM * 254) / 10;
         output_parameter.palette = palette;
@@ -998,7 +1115,7 @@ static int drv_1520_open(unsigned int prnr, unsigned int secondary)
 static void drv_1520_close(unsigned int prnr, unsigned int secondary)
 {
 #if DEBUG1520
-    log_message(drv1520_log, "drv_1520_close: sa=%d prnr=%d", secondary, prnr);
+    log_message(drv1520_log, "drv_1520_close: sa=%u prnr=%u", secondary, prnr);
 #endif
 
     /* Is this the last close? */
@@ -1008,6 +1125,9 @@ static void drv_1520_close(unsigned int prnr, unsigned int secondary)
 #if DEBUG1520
         log_message(drv1520_log, "drv_1520_close: last close");
 #endif
+        if (palette == NULL) {
+            log_message(drv1520_log, "PALETTE ALREADY DEALLOCATED!!\n");
+        }
         eject(mps);
 
         if (mps->sheet) {
@@ -1018,12 +1138,12 @@ static void drv_1520_close(unsigned int prnr, unsigned int secondary)
     }
 }
 
-static int drv_1520_putc(unsigned int prnr, unsigned int secondary, BYTE b)
+static int drv_1520_putc(unsigned int prnr, unsigned int secondary, uint8_t b)
 {
     plot_t *mps = &drv_1520[prnr];
 
 #if DEBUG1520
-    log_message(drv1520_log, "drv_1520_putc: sa=%d b='%c' (%d) prnr=%d", secondary, b, b, prnr);
+    log_message(drv1520_log, "drv_1520_putc: sa=%u b='%c' (%u) prnr=%u", secondary, b, b, prnr);
 #endif
 
     switch (secondary) {
@@ -1058,7 +1178,7 @@ static int drv_1520_putc(unsigned int prnr, unsigned int secondary, BYTE b)
     return 0;
 }
 
-static int drv_1520_getc(unsigned int prnr, unsigned int secondary, BYTE *b)
+static int drv_1520_getc(unsigned int prnr, unsigned int secondary, uint8_t *b)
 {
     return output_select_getc(prnr, b);
 }
@@ -1081,25 +1201,39 @@ static int drv_1520_formfeed(unsigned int prnr)
     if (mps->prnr == (int)prnr && mps->sheet != NULL) {
         eject(mps);
     }
+    output_select_formfeed(prnr);
+
     return 0;
 }
 
 int drv_1520_init_resources(void)
 {
-    driver_select_t driver_select;
+    driver_select_t driver;
 #if DEBUG1520
     log_message(drv1520_log, "drv_1520_init_resources");
 #endif
 
-    driver_select.drv_name = "1520";
-    driver_select.drv_open = drv_1520_open;
-    driver_select.drv_close = drv_1520_close;
-    driver_select.drv_putc = drv_1520_putc;
-    driver_select.drv_getc = drv_1520_getc;
-    driver_select.drv_flush = drv_1520_flush;
-    driver_select.drv_formfeed = drv_1520_formfeed;
+    driver = (driver_select_t){
+        .drv_name     = "1520",
+        .ui_name      = "Commodore 1520",
 
-    driver_select_register(&driver_select);
+        .drv_open     = drv_1520_open,
+        .drv_close    = drv_1520_close,
+        .drv_putc     = drv_1520_putc,
+        .drv_getc     = drv_1520_getc,
+        .drv_flush    = drv_1520_flush,
+        .drv_formfeed = drv_1520_formfeed,
+        .drv_select   = NULL,
+
+        .printer      = false,
+        .plotter      = true,
+        .iec          = true,
+        .ieee488      = false,
+        .userport     = false,
+        .text         = false,
+        .graphics     = true
+    };
+    driver_select_register(&driver);
 
     return 0;
 }
@@ -1123,9 +1257,9 @@ int drv_1520_init(void)
         return -1;
     }
 
-    if (palette_load("1520" FSDEV_EXT_SEP_STR "vpl", palette) < 0) {
+    if (palette_load("1520.vpl", "PRINTER", palette) < 0) {
         log_error(drv1520_log, "Cannot load palette file `%s'.",
-                  "1520" FSDEV_EXT_SEP_STR "vpl");
+                  "1520.vpl");
         return -1;
     }
 
@@ -1138,4 +1272,5 @@ void drv_1520_shutdown(void)
     log_message(drv1520_log, "drv_1520_shutdown");
 #endif
     palette_free(palette);
+    palette = NULL;
 }
