@@ -29,8 +29,7 @@
 
 struct snapshot_s;
 
-extern int tape_snapshot_write_module(struct snapshot_s *s, int save_image);
-extern int tape_snapshot_read_module(struct snapshot_s *s);
+int tape_snapshot_write_module(int port, struct snapshot_s *s, int save_image);
+int tape_snapshot_read_module(int port, struct snapshot_s *s);
 
 #endif
-
