@@ -24,10 +24,9 @@
  *
  */
 
-#ifndef _VIC20_CMDLINE_OPTIONS_H
-#define _VIC20_CMDLINE_OPTIONS_H
+#ifndef VICE_VIC20_CMDLINE_OPTIONS_H
+#define VICE_VIC20_CMDLINE_OPTIONS_H
 
-extern int vic20_cmdline_options_init(void);
+int vic20_cmdline_options_init(void);
 
 #endif
-
