@@ -1,5 +1,4 @@
-
-/*! \file vice.h 
+/*! \file vice.h
  *
  *  \brief Main header file for VICE.
  *
@@ -32,32 +31,17 @@
 #ifndef VICE_VICE_H
 #define VICE_VICE_H
 
-/* We use <config.h> instead of "config.h" so that a compilation using
-   -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
-   (which it would do because vice.h was found in $srcdir).  Well,
-   actually automake barfs if the source directory was already
-   configured, so this should not be an issue anymore.  */
-
-#ifdef IDE_COMPILE
-# include <ide-config.h> /* standard config file for IDE based compiles. */
-#else
-# include <config.h> /* Automagically created by the `configure' script.  */
-#endif
+#include "config.h" /* Automagically created by the `configure' script.  */
 
 /* ------------------------------------------------------------------------- */
 
 /* Portability... */
 
-#if defined(__hpux) || defined(__IBMC__)
-#ifndef _POSIX_SOURCE
-#define _POSIX_SOURCE
-#endif
-#ifndef _INCLUDE_POSIX_SOURCE
-#define _INCLUDE_POSIX_SOURCE
-#endif
-#endif  /* __hpux */
-
 /* currently tested/testing for the following cpu types:
+ *
+ * (please let's get rid of this, I personally enjoy making stuff work on OS's
+ * I don't expect to support it, but VICE should just support OS's people
+ * actually use. (Windows, Linux, MacOS, BSD)
  *
  * cpu        4*u_char fetch   1*u_int32 fetch   define(s)
  * -----      --------------   ---------------   ---------
@@ -101,70 +85,81 @@
 #define ALLOW_UNALIGNED_ACCESS
 #endif
 
-/* SunOS 4.x specific stuff */
-#if defined(sun) || defined(__sun)
-#  if !defined(__SVR4) && !defined(__svr4__)
-#    include <unistd.h>
-     typedef int ssize_t;
-#  endif
-#endif
-
-/* ------------------------------------------------------------------------- */
-/* Which OS is using the common keyboard routines?  */
-#if !defined(__OS2__) || defined(USE_SDLUI)
-#define COMMON_KBD
-#endif
-
-/* Which OS is using those ugly scale source coordinates.  */
-#if defined(__MSDOS__)
-#define VIDEO_SCALE_SOURCE
-#endif
-
 /* ------------------------------------------------------------------------- */
 
-/* Internationalization stuff */
-#if defined(ENABLE_NLS) && defined(HAVE_LIBINTL_H)
-#    include <libintl.h>
-#    define _(String) gettext (String)
-#    ifdef gettext_noop
-#        define N_(String) gettext_noop (String)
-#    else
-#        define N_(String) (String)
-#    endif
-#else
-/* Stubs that do something close enough.  */
-#    define _(String) (String)
-#    define N_(String) (String)
-#endif /* ENABLE_NLS */
-
-/* T_() is just an indicator for new common text which needs
-   to be added to the translate.* translation tables. */
-#define T_(String) (String)
-
-#ifdef __OS2__
-int yyparse (void);
-#undef __GNUC__
-#endif
-
-#ifdef MINIXVMD
-#undef vfork
-#endif
-
-#if (defined(__BEOS__) && defined(WORDS_BIGENDIAN)) || defined(__OS2__) || defined(__WATCOMC__)
+#if (defined(BEOS_COMPILE) && defined(WORDS_BIGENDIAN))
 #ifndef __cplusplus
 #undef inline
 #define inline
 #endif
 #endif
 
-/* interix using c89 doesn't like empty files, this will work around that */
-#if defined(_MSC_VER) && defined(__INTERIX)
-static int noop;
-#endif
-
 #ifdef USE_GCC
 #define int64_t_C(c) (c ## ll)
 #define uint64_t_C(c) (c ## ull)
 #endif
 
+/* Avoid windows.h including too much garbage
+ */
+#ifdef WINDOWS_COMPILE
+# define WIN32_LEAN_AND_MEAN
+#endif
+
+/* Provide define for checking 32/64-bit Windows
+ *
+ * No need for WIN32_COMPILE, just use !WIN64_COMPILE. Avoid confusion with the
+ * old WIN23_COMPILE define.
+ */
+#ifdef WINDOWS_COMPILE
+# ifdef _WIN64
+#  define WIN64_COMPILE
+# endif
+#endif
+
+/* some attribute defines that are useful mostly for static analysis */
+/* see https://clang.llvm.org/docs/AttributeReference.html */
+#ifdef __clang__
+#define VICE_ATTR_NORETURN  __attribute__((analyzer_noreturn))
+#elif defined(__GNUC__)
+#define VICE_ATTR_NORETURN  __attribute__((noreturn))
+#else
+#define VICE_ATTR_NORETURN
+#endif
+
+/* format checking attributes for printf style functions */
+#ifdef __clang__
+/* like regular printf, func(format, ...) */
+#define VICE_ATTR_PRINTF    __attribute__((__format__(__printf__, 1, 2)))
+/* one extra param on the left, func(param, format, ...) */
+#define VICE_ATTR_PRINTF2   __attribute__((__format__(__printf__, 2, 3)))
+/* two extra param on the left, func(param, param, format, ...) */
+#define VICE_ATTR_PRINTF3   __attribute__((__format__(__printf__, 3, 4)))
+/* three extra param on the left, func(param, param, param, format, ...) */
+#define VICE_ATTR_PRINTF4   __attribute__((__format__(__printf__, 4, 5)))
+/* one extra param after the format, func(format, param, ...) (used for resource sprintf) */
+#define VICE_ATTR_RESPRINTF __attribute__((__format__(__printf__, 1, 3)))
+#elif defined(__GNUC__)
+/* like regular printf, func(format, ...) */
+#define VICE_ATTR_PRINTF    __attribute__((format(printf, 1, 2)))
+/* one extra param on the left, func(param, format, ...) */
+#define VICE_ATTR_PRINTF2   __attribute__((format(printf, 2, 3)))
+/* two extra param on the left, func(param, param, format, ...) */
+#define VICE_ATTR_PRINTF3   __attribute__((format(printf, 3, 4)))
+/* three extra param on the left, func(param, param, param, format, ...) */
+#define VICE_ATTR_PRINTF4   __attribute__((format(printf, 4, 5)))
+/* one extra param after the format, func(format, param, ...) (used for resource sprintf) */
+#define VICE_ATTR_RESPRINTF __attribute__((format(printf, 1, 3)))
+#else
+#define VICE_ATTR_PRINTF
+#define VICE_ATTR_PRINTF2
+#define VICE_ATTR_PRINTF3
+#define VICE_ATTR_PRINTF4
+#define VICE_ATTR_RESPRINTF
+#endif
+
+/* M_PI is non-standard, so in order for -std=c99 to work we define it here */
+#ifndef M_PI
+#define M_PI 3.14159265358979323846
+#endif
+
 #endif
